@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKCLIENT_H
-#define GKCLIENT_H "@(#) $Id: GkClient.h,v 1.47 2012/12/10 19:22:22 shorne Exp $"
+#define GKCLIENT_H "@(#) $Id: GkClient.h,v 1.53 2013/05/01 19:35:45 shorne Exp $"
 
 #include "gk_const.h"
 #include "Toolkit.h"
@@ -35,12 +35,6 @@ class H225_Setup_UUIE;
 class H225_ArrayOf_ClearToken;
 class H225_ArrayOf_CryptoH323Token;
 
-#ifdef HAS_H46023
-  class H460_FeatureStd;
-  class STUNClient;
-  class UDPProxySocket;
-#endif
-
 class RasMsg;
 class RasServer;
 class AlternateGKs;
@@ -54,6 +48,36 @@ class SignalingMsg;
 template <class> class H225SignalingMsg;
 typedef H225SignalingMsg<H225_Setup_UUIE> SetupMsg;
 
+#ifdef HAS_H46023
+class H460_FeatureStd;
+class STUNClient;
+class UDPProxySocket;
+class H46024B_AlternateAddress;
+class H46024B_ArrayOf_AlternateAddress;
+class CallH46024Sockets
+{
+public:
+	CallH46024Sockets(unsigned strategy);
+	CallH46024Sockets(WORD sessionID, UDPProxySocket * rtp, UDPProxySocket * rtcp);
+	~CallH46024Sockets();
+
+	unsigned GetNatStrategy() const { return m_natStrategy; }
+	unsigned GetSessionID() const { return m_sessionID; }
+	void SetAlternate(PString cui, unsigned muxID, H323TransportAddress m_rtp, H323TransportAddress m_rtcp);
+	void SetAlternate(const H46024B_AlternateAddress & alternate);
+	void LoadAlternate(PString & cui, unsigned & muxID, H323TransportAddress & m_rtp, H323TransportAddress & m_rtcp);
+
+protected:
+	unsigned			m_natStrategy;
+	WORD				m_sessionID;
+	UDPProxySocket *	m_rtpSocket;
+	UDPProxySocket *	m_rtcpSocket;
+};
+
+typedef std::map<H225_CallIdentifier, std::list<CallH46024Sockets> > GkNATSocketMap;
+
+#endif
+
 class GkClient {
 public:
 	typedef GkClient Base;
@@ -64,12 +88,14 @@ public:
 	void OnReload();
 	void CheckRegistration();
 	bool CheckFrom(const RasMsg *) const;
-	bool CheckFrom(const PIPSocket::Address & ip) const { return m_gkaddr == ip; }
+	bool CheckFrom(const PIPSocket::Address & ip) const;
+	bool CheckFrom(const H225_TransportAddress & addr) const;
 	bool IsRegistered() const { return m_registered; }
 	bool IsNATed() const { return m_natClient != NULL; }
+	bool UsesH46018() const { return m_registeredH46018; }
 	PString GetParent() const;
 
-	bool UsesAdditiveRegistration();
+	bool UsesAdditiveRegistration() const;
 	bool AdditiveRegister(H225_ArrayOf_AliasAddress & aliases, int & rejectReason, 
 						H225_ArrayOf_ClearToken * tokens, H225_ArrayOf_CryptoH323Token * cryptotokens);
 	bool AdditiveUnRegister(const H225_ArrayOf_AliasAddress & aliases);
@@ -87,11 +113,14 @@ public:
 	
 	bool SendARQ(Routing::AdmissionRequest &);
 	bool SendLRQ(Routing::LocationRequest &);
-	bool SendARQ(Routing::SetupRequest &, bool answer = false, int natoffload = 0);
+	bool SendARQ(Routing::SetupRequest &, bool answer = false);
 	bool SendARQ(Routing::FacilityRequest &);
 	void SendDRQ(const callptr &);
 	void SendURQ();
 
+	// Signalling Messages
+	bool HandleSetup(SetupMsg & setup, bool fromInternal);
+
 	bool RewriteE164(H225_AliasAddress & alias, bool);
 	bool RewriteE164(H225_ArrayOf_AliasAddress & alias, bool);
 	bool RewriteE164(SetupMsg &setup, bool);
@@ -257,6 +286,11 @@ private:
 	H235Authenticators* m_h235Authenticators;
 #endif
 
+	// enable H.460.18 (offer to parent)
+	bool m_enableH46018;
+	// registered with H.460.18 support
+	bool m_registeredH46018;
+
 #ifdef HAS_H46023
 	// Handle H46023 RCF
 	void H46023_RCF(H460_FeatureStd * feat);
@@ -272,25 +306,38 @@ private:
 	int m_nattype;
 	// notify of NAT type
 	bool m_natnotify;
-	// H.460.23 support
-	bool gk_H460_23;
+	// enable H.460.23 (offer to parent)
+	bool m_enableH46023;
+	// registered with H.460.23 support
+	bool m_registeredH46023;
 	// STUN Client
 	STUNClient * m_stunClient;
 	// ALG Detected
 	bool m_algDetected;
     // Call NAT Strategy Map
 	PMutex m_strategyMutex;
-	std::map<H225_CallIdentifier, unsigned> m_natstrategy;
+	GkNATSocketMap m_natstrategy;
 
 public:
 	// NAT type detected
 	void H46023_TypeDetected(int nattype);
     // Create socket pair
-    bool H46023_CreateSocketPair(const H225_CallIdentifier & id, WORD flcn, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
+	bool H46023_CreateSocketPair(const H225_CallIdentifier & id, WORD sessionID, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
 	// Set the NAT Strategy
 	void H46023_SetNATStategy(const H225_CallIdentifier & id, unsigned nat);
 	// Find the NAT Strategy
 	CallRec::NatStrategy H46023_GetNATStategy(const H225_CallIdentifier & id);
+	// Remove the NAT Strategy
+	void H46023_RemoveNATStategy(const H225_CallIdentifier & id);
+	// Set the socketPair
+	void H46023_SetSocketPair(const H225_CallIdentifier & id, WORD sessionID, UDPProxySocket * rtp, UDPProxySocket * rtcp);
+	// Set Alternates (Annex A)
+	void H46023_SetAlternates(const H225_CallIdentifier & id, WORD session, PString cui, 
+							unsigned muxID, H323TransportAddress m_rtp, H323TransportAddress m_rtcp);
+	void H46023_LoadAlternates(const H225_CallIdentifier & id, WORD session, PString & cui, 
+							unsigned & muxID, H323TransportAddress & m_rtp, H323TransportAddress & m_rtcp);
+	// Set Alternates (Annex B)
+	void H46023_SetAlternates(const H225_CallIdentifier & id, const H46024B_ArrayOf_AlternateAddress & alternates);
 #endif
 };
 
