@@ -449,7 +449,7 @@ bool SSHStatusClient::Authenticate()
 						retries++;
                         if (AuthenticateUser()) {
 							auth = true;
-							ssh_message_auth_reply_success(message,0);
+							ssh_message_auth_reply_success(message, 0);
 							break;
 						}
 						// fall through: not authenticated, send default message
@@ -1345,7 +1345,7 @@ void StatusClient::DoDebug(
 			for (PINDEX i = 5; i < args.GetSize(); ++i)
 				value += " " + args[i];
 			Toolkit::Instance()->SetConfig(1, args[2], args[3], value);
-			WriteString(GkConfig()->GetString(args[2],args[3],"") + "\r\n");
+			WriteString(GkConfig()->GetString(args[2],args[3], "") + "\r\n");
 		} else if (args[1] *= "remove") {
 			if (args.GetSize() >= 4) {
 				Toolkit::Instance()->SetConfig(2, args[2], args[3]);
@@ -1773,13 +1773,14 @@ void StatusClient::ExecCommand(
 		WriteString("Output trace level is " + PString(m_traceLevel) + "\r\n");
 		break;
 
+#ifndef UNIT_TEST
 	case GkStatus::e_RotateLog:
 	    if (Gatekeeper::RotateLogFile())
 			WriteString("Log file rotation succeeded\r\n");
 		else						
 			WriteString("Log file rotation failed\r\n");
 	    break;
-				
+
 	case GkStatus::e_SetLogFilename:
 		if (args.GetSize() == 2) {
 			if (Gatekeeper::SetLogFilename(args[1])) {
@@ -1788,7 +1789,8 @@ void StatusClient::ExecCommand(
 				CommandError("Failed to open the log file'" + args[1] + "'");
 		} else
 			CommandError("Syntax Error: setlog <logfilepath>");
-		break;				
+		break;
+#endif
 
 	case GkStatus::e_AddIncludeFilter:
 	    if (args.GetSize() == 2) {
