@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKSTATUS_H
-#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.40 2011/10/18 19:24:51 willamowius Exp $"
+#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.43 2012/08/15 07:08:13 willamowius Exp $"
 
 #include <map>
 #include "yasocket.h"
@@ -79,6 +79,11 @@ public:
 		StatusClient* requestingClient
 		);
 
+
+	/** Notification form the status client that it has been deleted
+	*/
+	void StatusClientDeleted() { --m_statusClients; }
+
 	/** Print a list of all connected status interface users
 		to the requesting client.
 	*/
@@ -132,10 +137,8 @@ public:
 		e_BindAndRouteToGateway,       /// Route a call to a specified alias + destinationCallSignalAddr + specify bind IP
 		e_RouteReject,                 /// Reject to Route a call upon ARQ (send ARJ)
 		e_Trace,                       /// change trace level for status interface output
-#if PTRACING
 		e_RotateLog,                   /// Force log file rotation
 		e_SetLogFilename,              /// Change log file location
-#endif
 		e_AddIncludeFilter,            /// Add include filter
 		e_RemoveIncludeFilter,         /// Remove include filter
 		e_AddExcludeFilter,            /// Add exclude filter
@@ -150,6 +153,7 @@ public:
 		e_GetAcctInfo,                 /// Get acct module information
 		e_ResetCallCounters,           /// Reset the call counters
 		e_PrintEndpointQoS,            /// Print QoS values for all endpoints
+		e_PrintAllConfigSwitches,      /// print all known config switches
 		e_numCommands
 		/// Number of different strings
 	};
@@ -168,7 +172,7 @@ public:
 		PStringArray& args
 		);
 
-private:
+protected:
 	// override from class RegularJob
 	virtual void OnStart();
 
@@ -178,6 +182,9 @@ private:
 
 	/// map for fast (and easy) 'parsing' the commands from the user
 	std::map<PString, int> m_commands;
+
+	unsigned m_statusClients;
+	unsigned m_maxStatusClients;
 };
 
 /** Listen for incoming connections to the status interface port
