@@ -227,6 +227,12 @@ Neighbor::~Neighbor()
 
 bool Neighbor::SendLRQ(H225_RasMessage & lrq_ras)
 {
+	if (!m_sendPassword.IsEmpty()) {
+		H225_LocationRequest & lrq = lrq_ras;
+		lrq.IncludeOptionalField(H225_LocationRequest::e_cryptoTokens);
+		lrq.m_cryptoTokens.SetSize(1);
+		SetCryptoGkTokens(lrq.m_cryptoTokens, m_sendAuthUser, m_sendPassword);
+	}
 	return m_rasSrv->SendRas(lrq_ras, GetIP(), m_port);
 }
 
@@ -282,12 +288,6 @@ H225_LocationRequest & Neighbor::BuildLRQ(H225_RasMessage & lrq_ras, WORD seqnum
 		m_rasSrv->GetGkClient()->SetNBPassword(lrq, Toolkit::GKName());
 	}
 
-	if (!m_sendPassword.IsEmpty()) {
-		lrq.IncludeOptionalField(H225_LocationRequest::e_cryptoTokens);
-		lrq.m_cryptoTokens.SetSize(1);
-		SetCryptoGkTokens(lrq.m_cryptoTokens, m_sendAuthUser, m_sendPassword);
-	}
-
 	if (m_forwardHopCount >= 1) { // what if set hopCount = 1?
 		lrq.IncludeOptionalField(H225_LocationRequest::e_hopCount);
 		lrq.m_hopCount = m_forwardHopCount;
@@ -1540,9 +1540,9 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 
 bool NeighborPolicy::OnRequest(LocationRequest & lrq_obj)
 {
-	RasMsg *ras = lrq_obj.GetWrapper();
+	RasMsg * ras = lrq_obj.GetWrapper();
 	List::iterator iter = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsAcceptable), ras));
-	Neighbor *requester = (iter != m_neighbors.end()) ? *iter : NULL;
+	Neighbor * requester = (iter != m_neighbors.end()) ? *iter : NULL;
 	int hopCount = 0;
 	if (requester) {
 		if (requester->ForwardLRQ() < 0) {
@@ -1742,8 +1742,8 @@ SRVPolicy::SRVPolicy()
 	m_name = "SRV";
 	m_iniSection = "Routing::SRV";
 	m_resolveNonLocalLRQs = Toolkit::AsBool(GkConfig()->GetString(m_iniSection, "ResolveNonLocalLRQ", "0"));
-	m_ls_schema.SetAt("_h323ls._udp.","");
-	m_cs_schema.SetAt("_h323cs._tcp.","");
+	m_ls_schema.SetAt("_h323ls._udp.", "");
+	m_cs_schema.SetAt("_h323cs._tcp.", "");
 }
 
 void SRVPolicy::LoadConfig(const PString & instance)
@@ -1751,8 +1751,8 @@ void SRVPolicy::LoadConfig(const PString & instance)
 	if (instance.IsEmpty())
 		return;
 
-	m_ls_schema.SetSize(0);
-	m_cs_schema.SetSize(0);
+	m_ls_schema.RemoveAll();
+	m_cs_schema.RemoveAll();
 	m_cs_schema = GkConfig()->GetAllKeyValues(m_iniSection);
 }
 
@@ -1908,10 +1908,15 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly,
 				}
 
 				// If we have a gateway destination replace destination with our destination.
-				if (!gateway) 
-					dom = gateway;
+				PStringArray parts;
+				if (!gateway) {
+					parts = SplitIPAndPort(gateway, GK_DEF_ENDPOINT_SIGNAL_PORT);
+					PIPSocket::Address addr;
+					if (PIPSocket::GetHostAddress(parts[0], addr)) 
+						parts[0] = addr.AsString();
+				} else
+					parts = SplitIPAndPort(dom, GK_DEF_ENDPOINT_SIGNAL_PORT);
 
-				PStringArray parts = SplitIPAndPort(dom, GK_DEF_ENDPOINT_SIGNAL_PORT);
 				dom = parts[0];
 				WORD port = (WORD)parts[1].AsUnsigned();
 				PTRACE(4, "ROUTING\tSRV CS converted remote party " << alias << " to " << cs[j]);
@@ -1928,6 +1933,7 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly,
 						changed = true;
 					} else
 						H323SetAliasAddress(cs[j].Left(in), aliases[i]);
+
 					Route * route = NULL;
 					if (Toolkit::Instance()->IsGKHome(addr)) {
 						H225_ArrayOf_AliasAddress find_aliases;
@@ -1961,6 +1967,22 @@ bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 	bool localonly = dynamic_cast<LocationRequest *>(&request) && !m_resolveNonLocalLRQs;
 
 	Route * route = NULL;
+
+	// Always do the h323ls query first (if present)
+	for (PINDEX i = 0; i < m_ls_schema.GetSize(); ++i) {
+		PString ls_schema = m_ls_schema.GetKeyAt(i);
+		route = LSLookup(request, aliases, localonly, ls_schema);
+		if (route) {
+			if (route->m_flags & Route::e_Reject) {
+				request.SetFlag(RoutingRequest::e_Reject);
+			} else {
+				request.AddRoute(*route);
+			}
+			delete route;
+			return true;
+		}
+	}
+
 	for (PINDEX i = 0; i < m_cs_schema.GetSize(); ++i) {
 		PString cs_schema = m_cs_schema.GetKeyAt(i);
 		PStringArray dest = m_cs_schema.GetDataAt(i).Tokenise(",;");
@@ -1984,20 +2006,6 @@ bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 		}
 	}
 
-	if (m_ls_schema.GetSize() == 0)
-		return false;
-
-	PString ls_schema = m_cs_schema.GetKeyAt(0);
-	route = LSLookup(request, aliases, localonly, ls_schema);
-	if (route) {
-		if (route->m_flags & Route::e_Reject) {
-			request.SetFlag(RoutingRequest::e_Reject);
-		} else {
-			request.AddRoute(*route);
-		}
-		delete route;
-		return true;
-	}
 	return false;
 }
 
@@ -2059,7 +2067,7 @@ bool RDSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 	
 		// LS Record lookup
 		PStringList ls;
-		if (PDNS::RDSLookup(number,"H323+D2U",ls)) {
+		if (PDNS::RDSLookup(number, "H323+D2U", ls)) {
 			for (PINDEX i=0; i<ls.GetSize(); i++) {
 				PINDEX pos = ls[i].Find('@');
 				PString ipaddr = ls[i].Mid(pos + 1);
@@ -2129,8 +2137,110 @@ bool RDSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 }
 #endif
 
+
+class NeighborSqlPolicy : public SqlPolicy {
+public:
+	NeighborSqlPolicy();
+
+protected:
+	virtual bool ResolveRoute(RoutingRequest & request, DestinationRoutes & destination);
+};
+
+NeighborSqlPolicy::NeighborSqlPolicy()
+{
+	m_active = false;
+	m_sqlConn = NULL;
+#if HAS_DATABASE
+	m_name = "NeighborSql";
+	m_iniSection = "Routing::NeighborSql";
+	m_timeout = -1;
+#else
+	PTRACE(1, m_name << " not available - no database driver compiled into GnuGk");
+#endif // HAS_DATABASE
+}
+
+bool NeighborSqlPolicy::ResolveRoute(RoutingRequest & request, DestinationRoutes & destination)
+{
+		if (!destination.ChangeAliases()) {
+			PTRACE(2,  m_name << "\tNothing to resolve.");
+			return true;
+		}
+
+		if (destination.RejectCall()) {
+			destination.SetRejectReason(H225_AdmissionRejectReason::e_undefinedReason);
+			return false;
+		}
+
+		PString destinationIp = AsString(destination.GetNewAliases()[0],0);
+		PStringArray adr_parts = SplitIPAndPort(destinationIp,GK_DEF_UNICAST_RAS_PORT);
+		PIPSocket::Address ip;
+		if (!IsIPAddress(adr_parts[0]))
+		  PIPSocket::GetHostAddress(adr_parts[0],ip);
+		 else
+		  ip = adr_parts[0];
+		WORD port = (WORD)(adr_parts[1].AsInteger());
+		H323TransportAddress addr(ip,port);
+
+		if (addr == H323TransportAddress(RasServer::Instance()->GetRasAddress(ip))) {
+			PTRACE(2,  m_name << "\tERROR LRQ loop detected.");
+			return false;
+		}
+
+		// Create a gatekeeper object
+		GnuGK * nb = new GnuGK();
+		if (!nb->SetProfile(m_name, addr)) {
+			PTRACE(2, "ROUTING\tERROR setting " << m_name << " profile at " << addr);
+			SNMP_TRAP(11, SNMPError, Configuration, "Error setting " + PString(m_name) + " profile at " + AsString(addr));
+			delete nb;
+			destination.SetRejectReason(H225_AdmissionRejectReason::e_undefinedReason);
+			return false;
+		}
+
+		int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
+
+		// Send LRQ to retreive callers signaling address
+		// Caution: we may only use the functor object of the right type and never touch the others!
+		LRQSender<AdmissionRequest> ArqFunctor((AdmissionRequest &)request);
+		LRQSender<SetupRequest> SetupFunctor((SetupRequest &)request);
+		LRQSender<FacilityRequest> FacilityFunctor((FacilityRequest &)request);
+		LRQSender<LocationRequest> LrqFunctor((LocationRequest &)request);
+		LRQRequester * pRequest = NULL;
+		if (dynamic_cast<AdmissionRequest *>(&request)) {
+			pRequest = new LRQRequester(ArqFunctor);
+		} else if (dynamic_cast<SetupRequest *>(&request)) {
+			pRequest = new LRQRequester(SetupFunctor);
+		} else if (dynamic_cast<FacilityRequest *>(&request)) {
+			pRequest = new LRQRequester(FacilityFunctor);
+		} else if (dynamic_cast<LocationRequest *>(&request)) {
+			pRequest = new LRQRequester(LrqFunctor);
+		}
+		if (pRequest && pRequest->Send(nb)) {
+			if (H225_LocationConfirm * lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
+				Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_H460
+				if (pRequest->IsH46024Supported()) {
+					H225_RasMessage ras;
+					ras.SetTag(H225_RasMessage::e_locationConfirm);
+					H225_LocationConfirm & conf = (H225_LocationConfirm &)ras;
+					conf = *lcf;
+					route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+				}
+#endif
+				destination.AddRoute(route);
+				destination.SetChangedAliases(false);
+				delete pRequest;
+				delete nb;
+				return true;
+			}
+		}
+		delete pRequest;
+		delete nb;
+		return false;
+}
+
 namespace {
 	SimpleCreator<NeighborPolicy> NeighborPolicyCreator("neighbor");
+	SimpleCreator<NeighborSqlPolicy> NeighborSqlPolicyCreator("neighborsql");
 
 #ifdef hasSRV
 	SimpleCreator<SRVPolicy> SRVPolicyCreator("srv");
