@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef PROXYCHANNEL_H
-#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.128.2.2 2013/01/16 11:05:50 willamowius Exp $"
+#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.135 2013/02/28 23:38:57 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -335,7 +335,7 @@ public:
 	bool IsTraversalClient() const;
 	bool IsTraversalServer() const;
 	bool CreateRemote(const H225_TransportAddress & addr);
-	bool OnSCICall(H225_CallIdentifier callID, H225_TransportAddress sigAdr);
+	bool OnSCICall(const H225_CallIdentifier & callID, H225_TransportAddress sigAdr);
 	bool IsCallFromTraversalServer() const { return m_callFromTraversalServer; }
 	bool IsCallToTraversalServer() const { return m_callToTraversalServer; }
 	void SetSessionMultiplexDestination(WORD session, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
@@ -344,6 +344,10 @@ public:
 	void LockH245Handler() { m_h245handlerLock.Wait(); }
 	void UnlockH245Handler() { m_h245handlerLock.Signal(); }
 
+#ifdef HAS_H46018
+	bool IsH46019ClientCall(const H225_Setup_UUIE & setupBody);
+#endif
+
 #ifdef HAS_H46023
 	bool IsH46024Call(const H225_Setup_UUIE & setupBody);
 #endif
@@ -535,7 +539,7 @@ public:
 	bool sideBReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrB_RTCP) : IsSet(m_addrB); }
 	H46019Session SwapSides() const; // return a copy with side A and B swapped
 
-	bool IsKeepAlive(void * data, unsigned len, bool isRTCP) { return isRTCP ? true : (len == 12); };
+	bool IsKeepAlive(void * data, unsigned len, bool isRTCP) const { return isRTCP ? true : (len == 12); }
 
 	void HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
 	static void Send(PUInt32b sendMultiplexID, const H323TransportAddress & toAddress, int ossocket, void * data, unsigned len, bool bufferHasRoomForID=false);
@@ -610,7 +614,7 @@ public:
 
 protected:
 	MultiplexedRTPReader * m_reader;
-	mutable PReadWriteMutex listLock;
+	mutable PReadWriteMutex m_listLock;
 	list<H46019Session> m_h46019channels;
 	PUInt32b idCounter; // we should make sure this counter is _not_ reset on reload
 };
