@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASSRV_H
-#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.80 2012/11/26 14:47:12 willamowius Exp $"
+#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.84 2013/04/26 08:39:37 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -84,6 +84,7 @@ public:
 	bool IsGKRouted() const { return GKRoutedSignaling; }
 	bool IsH245Routed() const { return GKRoutedH245; }
 	bool AcceptUnregisteredCalls(const PIPSocket::Address &) const;
+	bool AcceptPregrantedCalls(const H225_Setup_UUIE & setupBody, const PIPSocket::Address &) const;
 	bool IsCallFromTraversalClient(const PIPSocket::Address &) const;
 	bool IsCallFromTraversalServer(const PIPSocket::Address &) const;
 
@@ -217,6 +218,9 @@ public:
 	template<class RAS> void SetAltGKInfo(RAS & ras, const NetworkAddress & ip)
 	{
 		H225_ArrayOf_AlternateGK alternates = GetAltGKForIP(ip);
+#if HAS_DATABASE
+		Toolkit::Instance()->AlternateGKs().GetAlternateGK(ip.m_address, alternates);
+#endif
 		if (alternates.GetSize() > 0) {
 			// add alternates by IP
 			ras.IncludeOptionalField(RAS::e_altGKInfo);
@@ -265,7 +269,7 @@ public:
 	virtual void Run();
 
 #ifdef HAS_H46017
-	virtual void ReadH46017Message(PBYTEArray ras, const PIPSocket::Address & fromIP, WORD fromPort, const PIPSocket::Address & localAddr, CallSignalSocket * s);
+	virtual void ReadH46017Message(const PBYTEArray & ras, const PIPSocket::Address & fromIP, WORD fromPort, const PIPSocket::Address & localAddr, CallSignalSocket * s);
 #endif
 
 private:
@@ -297,7 +301,7 @@ private:
 	PMutex requests_mutex;
 	std::list<RasMsg *> requests;
 	PMutex handlers_mutex;
-	std::list<RasHandler *> handlers;
+	std::list<RasHandler *> handlers;	// handlers checking for expected RAS messages eg. from parent or neighbors
 
 	bool GKRoutedSignaling, GKRoutedH245;
 	bool bRemoveCallOnDRQ;
