@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.249 2013/01/04 04:59:59 shorne Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.265 2013/05/01 19:35:46 shorne Exp $"
 
 #include <list>
 #include <map>
@@ -46,7 +46,7 @@ enum H46019Side { SideA, SideB };
 const int INVALID_OSSOCKET = -1;
 const WORD INVALID_RTP_SESSION = 0;
 const PUInt32b INVALID_MULTIPLEX_ID = 0;
-const BYTE GNUGK_KEEPALIVE_RTP_PAYLOADTYPE = 127;	// GnuGk always sends this fixed payload type
+const BYTE GNUGK_KEEPALIVE_RTP_PAYLOADTYPE = 116;	// must at least be 1 less than MAX_DYNAMIC_PAYLOAD_TYPE
 const BYTE MIN_DYNAMIC_PAYLOAD_TYPE = 96;
 const BYTE MAX_DYNAMIC_PAYLOAD_TYPE = 127;
 
@@ -172,6 +172,7 @@ public:
 	virtual bool LoadConfig();
 
 	virtual EndpointRec *Unregisterpreempt(int type);
+	virtual EndpointRec *Reregister();
 	virtual EndpointRec *Unregister();
 	virtual EndpointRec *Expired();
 
@@ -180,11 +181,13 @@ public:
 
 	virtual PString PrintOn(bool verbose) const;
 	PString PrintPrefixCapacities() const;
+	PString PrintNatInfo(bool verbose) const;
 
 	void SetNAT(bool nat);
 	void SetNATAddress(const PIPSocket::Address &, WORD port = 0);
 	void SetNATSocket(CallSignalSocket * socket);
 	void RemoveNATSocket();
+	void NullNATSocket();
 	void SetH46024(bool support);
 	void SetH46024A(bool support);
 	void SetH46024B(bool support);
@@ -238,9 +241,12 @@ public:
 
 	void SetUsesH460P(bool uses);
 	bool UsesH460P() const { return m_usesH460P; }
+	bool HasPresenceData();
 #ifdef HAS_H460P
 	void ParsePresencePDU(const PASN_OctetString & pdu);
+#if H460P_VER < 3
 	bool BuildPresencePDU(unsigned msgtag, PASN_OctetString & pdu);
+#endif
 #endif
 
 	/** If this Endpoint would be register itself again with all the same data
@@ -309,7 +315,7 @@ public:
 	void SetUsesH46026(bool val) {  m_usesH46026 = val; }
 	bool UsesH46026() const { return m_usesH46026; }
 	void SetTraversalRole(H46019TraversalType val) { m_traversalType = val; m_nat = (val == TraversalClient); }
-	H46019TraversalType GetTraversalRole() { return m_traversalType; }
+	H46019TraversalType GetTraversalRole() const { return m_traversalType; }
 	bool IsTraversalServer() const { return m_traversalType == TraversalServer; }
 	bool IsTraversalClient() const { return m_traversalType == TraversalClient; }
 
@@ -317,11 +323,11 @@ public:
 	typedef SmartPtr<EndpointRec> Ptr;
 
 protected:
-
 	void SetEndpointRec(H225_RegistrationRequest &);
 	void SetEndpointRec(H225_AdmissionRequest &);
 	void SetEndpointRec(H225_AdmissionConfirm &);
 	void SetEndpointRec(H225_LocationConfirm &);
+	void SetEndpointRec(H225_UnregistrationRequest &);	// used for temp objects
 
 	bool SendURQ(H225_UnregRequestReason::Choices, int preemption);
 
@@ -392,8 +398,9 @@ protected:
 	bool m_h46017disabled;
 	bool m_h46018disabled;
 	bool m_usesH460P;
+	bool m_hasH460PData;
 	bool m_usesH46017;
-    bool m_usesH46026;
+	bool m_usesH46026;
 	H46019TraversalType m_traversalType;	// this is not what GnuGk acts like, but what this EPRec is a proxy for
 	
 	long m_bandwidth;	// bandwidth currently occupied by this endpoint
@@ -553,10 +560,14 @@ public:
 		);
 
 	void ClearTable();
+	void UpdateTable();
 	void CheckEndpoints();
 
 	// handle remote closing of a NAT socket
 	void OnNATSocketClosed(CallSignalSocket * s);
+#ifdef HAS_H46017
+	void UnregisterAllH46017Endpoints();
+#endif
 
 	void PrintAllRegistrations(USocket *client, bool verbose=FALSE);
 	void PrintAllCached(USocket *client, bool verbose=FALSE);
@@ -593,7 +604,7 @@ private:
 	    //  or the stupid VC would fail to instantiate it
         	ReadLock lock(listLock);
         	const_iterator Iter(find_if(ListToBeFound->begin(), ListToBeFound->end(), FindObject));
-	        return endptr((Iter != ListToBeFound->end()) ? *Iter : 0);
+	        return endptr((Iter != ListToBeFound->end()) ? *Iter : NULL);
 	}
 
 	endptr InternalFindFirstEP(const H225_ArrayOf_AliasAddress & alias, std::list<EndpointRec *> *ListToBeFound);
@@ -645,6 +656,7 @@ public:
 	unsigned seq;
 	int ossocket;
 	PUInt32b multiplexID;
+	BYTE payloadType;
 	GkTimerManager::GkTimerHandle timer;
 };
 #endif
@@ -717,14 +729,9 @@ public:
 		);
 	
 	/// build a new call record with just a callID and transport adr (after H.460.18 SCI)
-	CallRec(
-		H225_CallIdentifier callID,
-		H225_TransportAddress sigAdr
-		);
+	CallRec(const H225_CallIdentifier & callID, H225_TransportAddress sigAdr);
  
-	CallRec(
-		CallRec *oldCall
-		);
+	CallRec(CallRec * oldCall);
 		
 	virtual ~CallRec();
 
@@ -737,6 +744,7 @@ public:
 
 	PINDEX GetCallNumber() const { return m_CallNumber; }
 	const H225_CallIdentifier & GetCallIdentifier() const { return m_callIdentifier; }
+	void ClearCallIdentifier() { m_callIdentifier = 0; }
 	unsigned GetCallRef() const { return m_crv; }
 	const H225_ConferenceIdentifier & GetConferenceIdentifier() const { return m_conferenceIdentifier; }
 	endptr GetCallingParty() const { return m_Calling; }
@@ -783,6 +791,10 @@ public:
 	  */
 	bool NATAssistCallerUnknown(NatStrategy & natinst);
 
+	/** Set Receive Side Strategy
+	  */
+	void SetReceiveNATStategy(NatStrategy & type, int & proxyMode);
+
 	/** Get String representation of the NATStrategy */
 	PString GetNATOffloadString(NatStrategy nat) const;
 
@@ -796,6 +808,19 @@ public:
 	/** Can Signalling be offloaded
 	  */
 	bool NATSignallingOffload(bool isAnswer) const;
+
+#ifdef HAS_H46024A
+    /** Whether Annex A passthrough
+      */
+    bool IsH46024APassThrough();
+
+	/** Send a H.460.24 Annex A indication
+	 */
+	PBoolean H46024AMessage();
+
+	/** GetSignallingSocket */
+	CallSignalSocket * H46024ASignalSocket();
+#endif
  
 #ifdef HAS_H46024B
     /** GetSignallingSocket */
@@ -809,6 +834,9 @@ public:
  
     /** Set session callback flag */
 	void H46024BSessionFlag(WORD sessionID);
+
+	/** Handle H46024B Request */
+	bool HandleH46024BRequest(const H245_ArrayOf_GenericParameter & content);
 #endif
  
 #endif
@@ -850,6 +878,7 @@ public:
 	void SetCallSignalSocketCalling(CallSignalSocket* socket);
 	void SetCallSignalSocketCalled(CallSignalSocket* socket);
 	void SetToParent(bool toParent) { m_toParent = toParent; }
+	void SetFromParent(bool fromParent) { m_fromParent = fromParent; }
 	void SetAccessTokens(const H225_ArrayOf_CryptoH323Token & tokens) { m_accessTokens = tokens; }
 	void SetInboundRewriteId(PString id) { m_inbound_rewrite_id = id; }
 	void SetOutboundRewriteId(PString id) { m_outbound_rewrite_id = id; }
@@ -881,7 +910,8 @@ public:
 
 	void SetH245Routed(PBoolean routed) { m_h245Routed = routed; }
 	bool IsH245Routed() const { return m_h245Routed; }
-	bool IsToParent() const { return m_toParent; }
+	bool IsToParent() const { return m_toParent; }	// to _or_ from parnet
+	bool IsFromParent() const { return m_fromParent; }	// from parent only
 	bool IsForwarded() const { return m_forwarded; }
 	bool IsSocketAttached() const { return (m_callingSocket != NULL); }
 
@@ -1185,16 +1215,14 @@ public:
 	    Calling party's aliases, as presented in ARQ or Setup messages.
 	    This does not change during the call.
 	*/
-	const H225_ArrayOf_AliasAddress& GetSourceAddress() const 
-		{ return m_sourceAddress; }
+	const H225_ArrayOf_AliasAddress GetSourceAddress() const { return m_sourceAddress; }
 		
 	/** @return
 	    Called party's aliases, as presented in ARQ or Setup messages.
 	    This does not change during the call now, but should be fixed
 	    to handle gatekeeper call forwarding properly.
 	*/
-	const H225_ArrayOf_AliasAddress& GetDestinationAddress() const
-		{ return m_destinationAddress; }
+	const H225_ArrayOf_AliasAddress GetDestinationAddress() const { return m_destinationAddress; }
 
 	/** @return
 	    Calling party's number or an empty string, if the number has not been
@@ -1215,7 +1243,7 @@ public:
 
 	/// Set call linkage This the party to be charged for the call. 
 	void SetCallLinkage(
-		const PString& id /// Calling-Station-Id (to be charged)
+		const PString & id /// Calling-Station-Id (to be charged)
 		);
 
 	/** @return
@@ -1225,7 +1253,7 @@ public:
 
 	/// Set calling party's number
 	void SetCalledStationId(
-		const PString& id /// Called-Station-Id
+		const PString & id /// Called-Station-Id
 		);
 
 	/** @return
@@ -1236,7 +1264,7 @@ public:
 
 	/// Set dialed number
 	void SetDialedNumber(
-		const PString& number /// Dialed-Number
+		const PString & number /// Dialed-Number
 		);
 
 	/** @return
@@ -1308,6 +1336,7 @@ public:
 	int GetH46019Direction() const;
 
 	void AddRTPKeepAlive(unsigned flcn, const H323TransportAddress & keepAliveRTPAddr, unsigned keepAliveInterval, PUInt32b multiplexID);
+	void SetRTPKeepAlivePayloadType(unsigned flcn, BYTE payloadType);
 	void StartRTPKeepAlive(unsigned flcn, int RTPOSSocket);
 	void AddRTCPKeepAlive(unsigned flcn, const H245_UnicastAddress & keepAliveRTCPAddr, unsigned keepAliveInterval, PUInt32b multiplexID);
 	void StartRTCPKeepAlive(unsigned flcn, int RTCPOSSocket);
@@ -1321,7 +1350,7 @@ public:
     typedef NATType EncDir;
     H235Authenticators & GetAuthenticators() { return m_authenticators; }
     void SetMediaEncryption(EncDir dir);
-    bool IsMediaEncryption() { return m_encyptDir != none; }
+    bool IsMediaEncryption() const { return m_encyptDir != none; }
     EncDir GetEncryptDirection() const { return m_encyptDir; }
 	BYTE GetNewDynamicPayloadType();
 #endif
@@ -1469,6 +1498,10 @@ private:
 #ifdef HAS_H46023
 	NatStrategy m_natstrategy;
 #endif
+
+#ifdef HAS_H46024A
+	void BuildH46024AnnexAIndication(H245_MultimediaSystemControlMessage & h245msg);
+#endif
  
 #ifdef HAS_H46024B
 	struct H46024Balternate {
@@ -1480,7 +1513,7 @@ private:
 	};
  
 	std::map<WORD,H46024Balternate> m_H46024Balternate;
-	void BuildH46024AnnexBMessage(bool initiate,H245_MultimediaSystemControlMessage & h245msg, const std::map<WORD,H46024Balternate> & alt);
+	void BuildH46024AnnexBRequest(bool initiate,H245_MultimediaSystemControlMessage & h245msg, const std::map<WORD,H46024Balternate> & alt);
 	list<int> m_h46024Bflag;
 #endif
 	/// unregistered caller NAT'd
@@ -1490,6 +1523,8 @@ private:
 	bool m_h245Routed;
 	/// the call is routed to this gatekeeper's parent gatekeeper
 	bool m_toParent;
+	/// the call is routed to this gatekeeper from it's parent gatekeeper
+	bool m_fromParent;
 	bool m_forwarded;
 	endptr m_Forwarder;
 
@@ -2073,7 +2108,7 @@ inline bool CallRec::IsTimeout(const time_t now) const
 class PreliminaryCall
 {
 public:
-	PreliminaryCall(CallSignalSocket * callerSocket, H225_CallIdentifier id, unsigned ref)
+	PreliminaryCall(CallSignalSocket * callerSocket, const H225_CallIdentifier & id, unsigned ref)
 		: m_socket(callerSocket), m_callid(id), m_callref(ref), m_proceedingSent(false) { }
 	~PreliminaryCall() { }
 
@@ -2098,8 +2133,8 @@ public:
 	~PreliminaryCallTable();
 	
 	void Insert(PreliminaryCall * call);
-	void Remove(H225_CallIdentifier id);
-	PreliminaryCall * Find(H225_CallIdentifier id) const;
+	void Remove(const H225_CallIdentifier & id);
+	PreliminaryCall * Find(const H225_CallIdentifier & id) const;
 
 private:
 	PreliminaryCallTable(const PreliminaryCallTable &);
