@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.78 2012/12/28 20:35:42 willamowius Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.80 2013/04/11 05:10:34 shorne Exp $"
 
 #include <map>
 #include <list>
@@ -122,6 +122,8 @@ public:
 	PString GetSourceIP() const { return m_sourceIP; }
 	void SetCallerID(const PString & id) { m_callerID = id; }
 	PString GetCallerID() const { return m_callerID; }
+	void SetGatewayDestination(const H225_TransportAddress & gw) { m_gwDestination = gw; }
+	bool GetGatewayDestination(H225_TransportAddress & gw ) const;
 
 private:
 	RoutingRequest(const RoutingRequest&);
@@ -134,6 +136,7 @@ private:
 	std::list<Route> m_failedRoutes;
 	PString m_sourceIP;
 	PString m_callerID;
+	H225_TransportAddress m_gwDestination;
 };
 
 template<class R, class W>
@@ -452,6 +455,7 @@ public:
 	unsigned int GetRejectReason() const { return m_rejectReason; }
 	void SetRejectReason(unsigned int reason) { m_rejectReason = reason; }
 	bool ChangeAliases() const { return m_aliasesChanged; }
+	void SetChangedAliases(bool success) { m_aliasesChanged = success; }
 	H225_ArrayOf_AliasAddress GetNewAliases() const { return m_newAliases; }
 	void SetNewAliases(const H225_ArrayOf_AliasAddress & aliases) { m_newAliases = aliases; m_aliasesChanged = true; }
 
@@ -493,6 +497,11 @@ protected:
 		/* out: */
 		DestinationRoutes & destination) = 0;
 
+	virtual bool ResolveRoute(
+		RoutingRequest & request, 
+		DestinationRoutes & destination
+		) { return true; }
+
 protected:
 	// active ?
 	bool m_active;
