@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.102 2012/11/29 14:29:16 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.106 2013/04/17 21:11:50 willamowius Exp $"
 
 #include <vector>
 #include <map>
@@ -44,14 +44,9 @@ public:
 	/// Build an "any" address
 	NetworkAddress();
 	/// Build a single host address
-	NetworkAddress(
-		const PIPSocket::Address &addr
-		);
+	NetworkAddress(const PIPSocket::Address & addr);
 	/// Build an address of the specified network
-	NetworkAddress(
-		const PIPSocket::Address &addr,
-		const PIPSocket::Address &nm
-		);
+	NetworkAddress(const PIPSocket::Address & addr, const PIPSocket::Address & nm);
 	/// Build an address from the string
 	NetworkAddress(
 		const PString &str /// an address in a form A.B.C.D, A.B.C.D/24 or A.B.C.D/255.255.255.0
@@ -127,7 +122,7 @@ class Toolkit : public Singleton<Toolkit>
 		virtual ~RouteTable() { ClearTable(); }
 		Address GetLocalAddress(unsigned version) const;
 		Address GetLocalAddress(const Address & dest) const;
-		Address GetDefaultIP(unsigned version);
+		Address GetDefaultIP(unsigned version) const;
 
 		void InitTable();
 		void ClearTable();
@@ -196,7 +191,7 @@ class Toolkit : public Singleton<Toolkit>
 	int SelectRoutingMode(const PIPSocket::Address & ip1, const PIPSocket::Address & ip2) const
 	{ return m_ProxyCriterion.SelectRoutingMode(ip1, ip2); }
 
-	bool IsInternal(const PIPSocket::Address & ip1)
+	bool IsInternal(const PIPSocket::Address & ip1) const
 	{ return (m_ProxyCriterion.IsInternal(ip1) > 0); }
 
 	// Since PStringToString is not thread-safe,
@@ -260,7 +255,7 @@ class Toolkit : public Singleton<Toolkit>
 #endif
 	};
 
-	AssignedAliases GetAssignedEPAliases() { return m_AssignedEPAliases; }
+	AssignedAliases GetAssignedEPAliases() const { return m_AssignedEPAliases; }
 
 #ifdef h323v6
 	class AssignedGatekeepers {
@@ -295,7 +290,7 @@ class Toolkit : public Singleton<Toolkit>
 
 	};
 
-	AssignedGatekeepers AssignedGKs() { return m_AssignedGKs; }
+	AssignedGatekeepers AssignedGKs() const { return m_AssignedGKs; }
 #endif
 
 #if HAS_DATABASE
@@ -317,7 +312,7 @@ class Toolkit : public Singleton<Toolkit>
 	   // query timeout
 	   long m_timeout;
 	};
-	AlternateGatekeepers AlternateGKs() { return m_AlternateGKs; }
+	AlternateGatekeepers AlternateGKs() const { return m_AlternateGKs; }
 #endif
 
 #if HAS_DATABASE
@@ -339,7 +334,7 @@ class Toolkit : public Singleton<Toolkit>
 	   long m_timeout;
 	};
 
-	QoSMonitor QoS() { return m_qosMonitor; }
+	QoSMonitor QoS() const { return m_qosMonitor; }
 #endif
 
 
@@ -352,7 +347,7 @@ class Toolkit : public Singleton<Toolkit>
 	bool RewriteE164(H225_AliasAddress & alias);
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
 
-	bool RewritePString(PString & s) { return m_Rewrite.RewritePString(s); }
+	bool RewritePString(PString & s) const { return m_Rewrite.RewritePString(s); }
 
 	// Class to allow correct use of STL inside PDictionary type
 	class GWRewriteEntry : public PObject {
@@ -395,7 +390,7 @@ class Toolkit : public Singleton<Toolkit>
 #endif
 
 	bool isBehindNAT(PIPSocket::Address &) const;
-	std::vector<NetworkAddress> GetInternalNetworks();
+	std::vector<NetworkAddress> GetInternalNetworks() const;
 #ifdef HAS_H46018
 	bool IsH46018Enabled() const { return m_H46018Enabled; }
 #endif
@@ -560,7 +555,7 @@ class Toolkit : public Singleton<Toolkit>
 	PString AsString(
 		const PTime& tm, /// timestamp to convert into a string
 		const PString& formatStr = PString::Empty() /// format string to use
-		);
+		) const;
 
 	/** Read and decrypt a password from the config. As a decryption key
 	    this function is using the given key name padded with bytes of value
@@ -604,6 +599,9 @@ class Toolkit : public Singleton<Toolkit>
 	/// global translation of cause codes (called from endpoint specific method)
 	unsigned TranslateReceivedCause(unsigned cause) const;
 	unsigned TranslateSentCause(unsigned cause) const;
+	// check if cause contranslation feature is active (global or per endpoint)
+	bool IsCauseCodeTranslationActive() const { return m_causeCodeTranslationActive; }
+	void SetCauseCodeTranslationActive(bool val) { m_causeCodeTranslationActive = val; }
 
 #ifdef OpenH323Factory
 	PStringList GetAuthenticatorList();
@@ -656,7 +654,7 @@ private:
 	/// if true, all passwords in the config are encrypted
 	bool m_encryptAllPasswords;
 	/// set of ANI/CLI rewrite rules
-	CLIRewrite *m_cliRewrite;
+	CLIRewrite * m_cliRewrite;
 	/// bit flag failover triggers for 128 Q931 causes
 	unsigned char m_causeMap[16];
 	/// map H.225 reason to Q.931 cause code
@@ -664,6 +662,7 @@ private:
 	/// global cause code translation
 	std::map<unsigned, unsigned> m_receivedCauseMap;
 	std::map<unsigned, unsigned> m_sentCauseMap;
+	bool m_causeCodeTranslationActive;	// globally _or_ per endpoint
 	bool m_alwaysRemoveH235Tokens;
 	std::vector<NetworkAddress> m_removeH235TokensfromNetwork;
 #ifdef HAS_H235_MEDIA
