@@ -27,6 +27,7 @@
 #include "stl_supp.h"
 #include "RasSrv.h"
 #include "RasTbl.h"
+#include "ProxyChannel.h"
 #include "SoftPBX.h"
 #include "MakeCall.h"
 #include "gktimer.h"
@@ -39,10 +40,6 @@ using std::vector;
 
 PCREATE_PROCESS(Gatekeeper)
 
-PIPSocket::Address GNUGK_INADDR_ANY(INADDR_ANY);
-
-PReadWriteMutex ConfigReloadMutex;
-
 PTextFile* Gatekeeper::m_logFile = NULL;
 PFilePath Gatekeeper::m_logFilename;
 PMutex Gatekeeper::m_logFileMutex;
@@ -52,10 +49,9 @@ int Gatekeeper::m_rotateHour = 0;
 int Gatekeeper::m_rotateDay = 0;
 GkTimer* Gatekeeper::m_rotateTimer = GkTimerManager::INVALID_HANDLE;
 
-PSemaphore ShutdownMutex(1,1);
-
 const char * KnownConfigEntries[][2] = {
 	// valid config entries
+#ifdef HAS_DATABASE
 	{ "AlternateGatekeepers::SQL", "CacheTimeout" },
 	{ "AlternateGatekeepers::SQL", "Database" },
 	{ "AlternateGatekeepers::SQL", "Driver" },
@@ -83,6 +79,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "AssignedGatekeepers::SQL", "Password" },
 	{ "AssignedGatekeepers::SQL", "Query" },
 	{ "AssignedGatekeepers::SQL", "Username" },
+#endif
 	{ "CTI::Agents", "CTI_Timeout" },	// obsolete, but still accepted
 	{ "CTI::Agents", "RequestTimeout" },
 	{ "CTI::Agents", "VirtualQueue" },	// obsolete, but still accepted
@@ -180,6 +177,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "Gatekeeper::Main", "Home" },
 	{ "Gatekeeper::Main", "ListenQueueLength" },
 	{ "Gatekeeper::Main", "MaxASNArraySize" },
+	{ "Gatekeeper::Main", "MaxStatusClients" },
 	{ "Gatekeeper::Main", "MaximumBandwidthPerCall" },
 	{ "Gatekeeper::Main", "MinimumBandwidthPerCall" },
 	{ "Gatekeeper::Main", "MulticastGroup" },
@@ -213,6 +211,7 @@ const char * KnownConfigEntries[][2] = {
 #ifdef hasLDAPStartTLS
 	{ "GkH350::Settings", "StartTLS" },
 #endif
+#ifdef HAS_DATABASE
 	{ "GkPresence::SQL", "CacheTimeout" },
 	{ "GkPresence::SQL", "Database" },
 	{ "GkPresence::SQL", "Driver" },
@@ -227,9 +226,11 @@ const char * KnownConfigEntries[][2] = {
 	{ "GkPresence::SQL", "QueryUpdate" },
 	{ "GkPresence::SQL", "UpdateWorkerTimer" },
 	{ "GkPresence::SQL", "Username" },
+#endif
 	{ "GkQoSMonitor", "CallEndOnly" },
 	{ "GkQoSMonitor", "DetailFile" },
 	{ "GkQoSMonitor", "Enable" },
+#ifdef HAS_DATABASE
 	{ "GkQoSMonitor::SQL", "CacheTimeout" },
 	{ "GkQoSMonitor::SQL", "Database" },
 	{ "GkQoSMonitor::SQL", "Driver" },
@@ -238,10 +239,13 @@ const char * KnownConfigEntries[][2] = {
 	{ "GkQoSMonitor::SQL", "Password" },
 	{ "GkQoSMonitor::SQL", "Query" },
 	{ "GkQoSMonitor::SQL", "Username" },
+#endif
 	{ "GkStatus::Filtering", "Enable" },
 	{ "GkStatus::Filtering", "ExcludeFilter" },
 	{ "GkStatus::Filtering", "IncludeFilter" },
 	{ "GkStatus::Filtering", "NewRCFOnly" },
+	{ "GkStatus::Message", "Compact" },
+	{ "GkStatus::Message", "RCF" },
 	{ "LogFile", "Filename" },
 	{ "LogFile", "Rotate" },
 	{ "LogFile", "RotateDay" },
@@ -269,6 +273,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "Proxy", "ProxyAlways" },
 	{ "Proxy", "ProxyForNAT" },
 	{ "Proxy", "ProxyForSameNAT" },
+	{ "Proxy", "RTPDiffServ" },
 	{ "Proxy", "RTPMultiplexing" },
 	{ "Proxy", "RTPMultiplexPort" },
 	{ "Proxy", "RTCPMultiplexPort" },
@@ -320,7 +325,6 @@ const char * KnownConfigEntries[][2] = {
 	{ "RadAuth", "SocketDeleteTimeout" },
 	{ "RadAuth", "UseDialedNumber" },
 	{ "RasSrv::ARQFeatures", "ArjReasonRouteCallToGatekeeper" },
-	{ "RasSrv::ARQFeatures", "CallUnregisteredEndpoints" },
 	{ "RasSrv::ARQFeatures", "RemoveTrailingChar" },
 	{ "RasSrv::ARQFeatures", "RoundRobinGateways" },
 	{ "RasSrv::ARQFeatures", "SendRIP" },
@@ -332,6 +336,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "RasSrv::LRQFeatures", "ForwardLRQ" },
 	{ "RasSrv::LRQFeatures", "ForwardResponse" },
 	{ "RasSrv::LRQFeatures", "NeighborTimeout" },
+	{ "RasSrv::LRQFeatures", "PingAlias" },
 	{ "RasSrv::LRQFeatures", "SendRetries" },
 	{ "RasSrv::LRQFeatures", "SendRIP" },
 	{ "RasSrv::RRQFeatures", "AcceptEndpointIdentifier" },
@@ -339,10 +344,30 @@ const char * KnownConfigEntries[][2] = {
 	{ "RasSrv::RRQFeatures", "AcceptMCUPrefixes" },
 	{ "RasSrv::RRQFeatures", "AccHTTPLink" },
 	{ "RasSrv::RRQFeatures", "AliasTypeFilter" },
+	{ "RasSrv::RRQFeatures", "AuthenticatedAliasesOnly" },
+	{ "RasSrv::RRQFeatures", "EnableAdditiveRegistration" },
+	{ "RasSrv::RRQFeatures", "GatewayAssignAliases" },
 	{ "RasSrv::RRQFeatures", "IRQPollCount" },
 	{ "RasSrv::RRQFeatures", "OverwriteEPOnSameAddress" },
-	{ "RasSrv::RRQFeatures", "EnableAdditiveRegistration" },
 	{ "RasSrv::RRQFeatures", "SupportDynamicIP" },
+#ifdef HAS_DATABASE
+	{ "RewriteCLI::SQL", "CacheTimeout" },
+	{ "RewriteCLI::SQL", "Database" },
+	{ "RewriteCLI::SQL", "Driver" },
+	{ "RewriteCLI::SQL", "Host" },
+	{ "RewriteCLI::SQL", "InboundQuery" },
+	{ "RewriteCLI::SQL", "Library" },
+	{ "RewriteCLI::SQL", "MinPoolSize" },
+	{ "RewriteCLI::SQL", "OutboundQuery" },
+	{ "RewriteCLI::SQL", "Password" },
+	{ "RewriteCLI::SQL", "Username" },
+#endif
+	{ "RewriteSourceAddress", "ForceAliasType" },
+	{ "RewriteSourceAddress", "MatchSourceTypeToDestination" },
+	{ "RewriteSourceAddress", "OnlyE164" },
+	{ "RewriteSourceAddress", "OnlyValid10Dand11D" },
+	{ "RewriteSourceAddress", "ReplaceChar" },
+	{ "RewriteSourceAddress", "Rules" },
 	{ "RoutedMode", "AcceptNeighborCalls" },
 	{ "RoutedMode", "AcceptNeighborsCalls" },
 	{ "RoutedMode", "AcceptUnregisteredCalls" },
@@ -407,6 +432,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "RoutedMode", "RemoveCallOnDRQ" },
 	{ "RoutedMode", "RemoveFaxUDPOptionsFromRM" },
 	{ "RoutedMode", "RemoveH235Call" },
+	{ "RoutedMode", "RemoveH245AddressFromSetup" },
 	{ "RoutedMode", "RemoveH245AddressOnTunneling" },
 	{ "RoutedMode", "RemoveH460Call" },
 	{ "RoutedMode", "RemoveSorensonSourceInfo" },
@@ -448,8 +474,18 @@ const char * KnownConfigEntries[][2] = {
 	{ "Routing::Lua", "Script" },
 	{ "Routing::Lua", "ScriptFile" },
 #endif
+	{ "Routing::NeighborSql", "CacheTimeout" },
+	{ "Routing::NeighborSql", "Database" },
+	{ "Routing::NeighborSql", "Driver" },
+	{ "Routing::NeighborSql", "Host" },
+	{ "Routing::NeighborSql", "Library" },
+	{ "Routing::NeighborSql", "MinPoolSize" },
+	{ "Routing::NeighborSql", "Password" },
+	{ "Routing::NeighborSql", "Query" },
+	{ "Routing::NeighborSql", "Username" },
 	{ "Routing::RDS", "ResolveLRQ" },
 	{ "Routing::SRV", "ResolveNonLocalLRQ" },
+#ifdef HAS_DATABASE
 	{ "Routing::Sql", "CacheTimeout" },
 	{ "Routing::Sql", "Database" },
 	{ "Routing::Sql", "Driver" },
@@ -460,6 +496,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "Routing::Sql", "Password" },
 	{ "Routing::Sql", "Query" },
 	{ "Routing::Sql", "Username" },
+#endif
 #ifdef HAS_SNMP
 	{ "SNMP", "AllowRequestsFrom" },
 	{ "SNMP", "AgentListenIP" },
@@ -470,6 +507,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "SNMP", "TrapCommunity" },
 	{ "SNMP", "TrapHost" },
 #endif
+#ifdef HAS_DATABASE
 	{ "SQLAcct", "CacheTimeout" },
 	{ "SQLAcct", "AlertQuery" },
 	{ "SQLAcct", "Database" },
@@ -519,6 +557,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "SQLConfig", "Library" },
 	{ "SQLConfig", "MinPoolSize" },
 	{ "SQLConfig", "NeighborsQuery" },
+	{ "SQLConfig", "NeighborsQuery2" },
 	{ "SQLConfig", "Password" },
 	{ "SQLConfig", "PermanentEndpointsQuery" },
 	{ "SQLConfig", "RewriteAliasQuery" },
@@ -533,6 +572,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "SQLPasswordAuth", "Password" },
 	{ "SQLPasswordAuth", "Query" },
 	{ "SQLPasswordAuth", "Username" },
+#endif
 	{ "StatusAcct", "ConnectEvent" },
 	{ "StatusAcct", "RegisterEvent" },
 	{ "StatusAcct", "StartEvent" },
@@ -623,6 +663,14 @@ PTimedMutex ReloadMutex;
 PString pidfile("/var/run/gnugk.pid");
 #endif
 
+#ifdef P_LINUX
+PString RlimAsString(unsigned long rlim_val)
+{
+	return (rlim_val == RLIM_INFINITY) ? PString("unlimited") : PString(rlim_val);
+
+}
+#endif
+
 void ShutdownHandler()
 {
 	Gatekeeper::EnableLogFileRotation(false);
@@ -630,8 +678,14 @@ void ShutdownHandler()
 	PTRACE(3, "GK\tDeleting global reference tables");
 
 	Job::StopAll();
+#ifdef HAS_H46018
+	if (MultiplexedRTPHandler::InstanceExists())
+		delete MultiplexedRTPHandler::Instance();
+#endif
 	if (CapacityControl::InstanceExists())
 		delete CapacityControl::Instance();
+	if (PreliminaryCallTable::InstanceExists())
+		delete PreliminaryCallTable::Instance();
 	if (CallTable::InstanceExists())
 		delete CallTable::Instance();
 	if (RegistrationTable::InstanceExists())
@@ -749,10 +803,10 @@ void ReloadHandler()
 	Gatekeeper::ReopenLogFile();
 
 	// only one thread must do this
-#if PTLIB_VER < 2120
-	if (ReloadMutex.WillBlock())
-#else
+#ifdef hasNoMutexWillBlock
 	if (!ReloadMutex.Try())
+#else
+	if (ReloadMutex.WillBlock())
 #endif
 		return;
 
@@ -868,7 +922,7 @@ BOOL SetDEP(__in DWORD dwFlags = PROCESS_DEP_ENABLE)
 	if (!hMod)
 		return FALSE;
 	typedef BOOL (WINAPI *PSETDEP)(DWORD);
-	PSETDEP procSet = (PSETDEP)GetProcAddress(hMod,"SetProcessDEPPolicy");
+	PSETDEP procSet = (PSETDEP)GetProcAddress(hMod, "SetProcessDEPPolicy");
 	if (!procSet)
 		return FALSE;
 	return procSet(dwFlags);
@@ -906,6 +960,10 @@ void UnixShutdownHandler(int sig)
 		return;
 	PWaitAndSignal shutdown(ShutdownMutex);
 	PTRACE(1, "GK\tReceived signal " << sig);
+#ifdef HAS_H46017
+	// unregister all H.460.17 endpoints before we stop the socket handlers and thus delete their sockets
+	RegistrationTable::Instance()->UnregisterAllH46017Endpoints();
+#endif
 	PFile::Remove(pidfile);
 	RasServer::Instance()->Stop();
 }
@@ -950,7 +1008,7 @@ Gatekeeper::Gatekeeper(const char * _manuf,
 	GetArguments().SetArgs("-d -p " + pidfile);
 #endif
 #endif
-#if (PTLIB_VER >= 2100) && (PTLIB_VER < 2120)
+#ifdef hasThreadAutoDeleteBug
 	// work around a bug in PTLib 2.10.x that doesn't start the housekeeping thread to delete auto-delete threads
 	SignalTimerChange();
 #endif
@@ -1153,25 +1211,40 @@ void Gatekeeper::Main()
 #ifdef P_LINUX
 	// set the core file size
 	if (args.HasOption("core")) {
+		PString msg;
 		struct rlimit rlim;
-		if (getrlimit(RLIMIT_CORE, &rlim) != 0)
-			cout << "Could not get current core file size : error = " << errno << endl;
-		else {
-			cout << "Current core dump size limits - soft: " << rlim.rlim_cur
-				<< ", hard: " << rlim.rlim_max << endl;
+		if (getrlimit(RLIMIT_CORE, &rlim) != 0) {
+			msg = "Error: Could not get current core file size : error = " + PString(errno);
+			cout << msg << endl;
+			PTRACE(1, msg);
+		} else {
+			msg = "Current core dump size limits - soft: " + RlimAsString(rlim.rlim_cur) + ", hard: " + RlimAsString(rlim.rlim_max);
+			cout << msg << endl;
+			PTRACE(3, msg);
 			int uid = geteuid();
-			seteuid(getuid()); // Switch back to starting uid for next call
+			int result = seteuid(getuid()); // switch back to starting uid for next call
+			if (result != 0) {
+				PTRACE(1, "Warning: Setting EUID failed");
+			}
 			const PCaselessString s = args.GetOptionString("core");
 			rlim_t v = (s == "unlimited" ? RLIM_INFINITY : (rlim_t)s.AsInteger());
 			rlim.rlim_cur = v;
-			if (setrlimit(RLIMIT_CORE, &rlim) != 0)
-				cout << "Could not set current core file size to " << v << " : error = " << errno << endl;
-			else {
+			if (rlim.rlim_max < rlim.rlim_cur)
+				rlim.rlim_max = v;
+			if (setrlimit(RLIMIT_CORE, &rlim) != 0) {
+				msg = "Error: Could not set current core file size to " + RlimAsString(v) + " : error = " + PString(errno);
+				cout << msg << endl;
+				PTRACE(1, msg);
+			} else {
 				getrlimit(RLIMIT_CORE, &rlim);
-				cout << "New core dump size limits - soft: " << rlim.rlim_cur
-					<< ", hard: " << rlim.rlim_max << endl;
+				msg = "New core dump size limits - soft: " + RlimAsString(rlim.rlim_cur) + ", hard: " + RlimAsString(rlim.rlim_max);
+				cout << msg << endl;
+				PTRACE(3, msg);
+			}
+			result = seteuid(uid);
+			if (result != 0) {
+				PTRACE(1, "Warning: Setting EUID failed");
 			}
-			seteuid(uid);
 		}
 	}
 #endif
@@ -1213,7 +1286,7 @@ void Gatekeeper::Main()
 			fake_cmdline += " -o " + log_trace_file;
 	}
 	if (!fake_cmdline.IsEmpty()) {
-		for (int t=0; t < args.GetOptionCount('t'); t++)
+		for (int t = 0; t < args.GetOptionCount('t'); t++)
 			fake_cmdline += " -t";
 		PArgList fake_args(fake_cmdline);
 		fake_args.Parse(GetArgumentsParseString());
@@ -1226,6 +1299,8 @@ void Gatekeeper::Main()
 	cout << welcome << '\n';
 	PTRACE(1, welcome);
 
+	PTRACE(1, "Current file handle limit: " << PProcess::Current().GetMaxHandles());
+
 #ifdef hasIPV6
 	if (Toolkit::Instance()->IsIPv6Enabled()) {
 		PTRACE(1, "IPv6 enabled");
