@@ -4,7 +4,7 @@
  * PostgreSQL driver module for GnuGk
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
- * Copyright (c) 2006-2011, Jan Willamowius
+ * Copyright (c) 2006-2012, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -354,6 +354,7 @@ GkSQLConnection::SQLConnPtr GkPgSQLConnection::CreateNewConnection(
 			PTRACE (1, GetName() << "\tFailed to load shared database library: unknown error");
 #endif
 			g_sharedLibrary.Close();
+			SNMP_TRAP(5, SNMPError, Database, GetName() + " DLL load error");
 			return NULL;
 		}
 	}
@@ -374,8 +375,8 @@ GkSQLConnection::SQLConnPtr GkPgSQLConnection::CreateNewConnection(
 	} else {
 		PTRACE(2, GetName() << "\tPgSQL connection to " << m_username << '@' << m_host 
 			<< '[' << m_database << "] failed (PQsetdbLogin failed): " 
-			<< (conn ? (*g_PQerrorMessage)(conn) : "")
-			);
+			<< (conn ? (*g_PQerrorMessage)(conn) : ""));
+		SNMP_TRAP(5, SNMPError, Database, GetName() + " connection failed")
 		if (conn)
 			(*g_PQfinish)(conn);
 	}
