@@ -2,7 +2,7 @@
 //
 // H.323 utility functions
 //
-// Copyright (c) 2000-2012, Jan Willamowius
+// Copyright (c) 2000-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -52,7 +52,7 @@ PString AsString(const PIPSocket::Address & ip)
 		sockaddr_in6 ipv6addr;
 		memset(&ipv6addr, 0, sizeof(ipv6addr));
 		ipv6addr.sin6_addr = ip;
-		return inet_ntop(AF_INET6, &ipv6addr.sin6_addr, buf, INET6_ADDRSTRLEN);
+		return inet_ntop(AF_INET6, (const void *)&ipv6addr.sin6_addr, buf, INET6_ADDRSTRLEN);
 	}
 #endif
 	return ip.AsString();
@@ -200,6 +200,11 @@ PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasType)
 				if (prefixIndex != P_MAX_INDEX)
 					aliasString = aliasString.Mid(prefixIndex + 1);
 			}
+			// simplify URL aliases to match "h323:foo@bar" with "foo@bar"
+			if ((terminalAlias.GetTag() == H225_AliasAddress::e_url_ID)
+				&& (aliasString.Left(5) == "h323:")) {
+				aliasString = aliasString.Mid(5);
+			}
 			if (includeAliasType) {
 				aliasString += ":" + terminalAlias.GetTagName();
 			}
@@ -734,10 +739,7 @@ PString RewriteString(
 	return result;
 }
 
-PString RewriteWildcard(
-	const PString & s,
-	const PString & expression 
-	)
+PString RewriteWildcard(const PString & s, const PString & expression)
 {
 	PString o = expression;
 
@@ -750,15 +752,16 @@ PString RewriteWildcard(
 	int b2 = o.Find('}');
 	if (b2 == P_MAX_INDEX || b1 > b2) return s;  /// Logic error ignore rewrite
 
+	PString l = o.Left(b1);
+	PString r = o.Mid(b2+1);
 	PString exp = o.Mid(b1+1,b2-b1-1);
-
-	PString r;
+	PString rewrite;
 	if (exp *= "\\1")
-		r = s;
+		rewrite = s;
 	else {
 	   int start = -1;
-       if (exp.Find('^') != P_MAX_INDEX)  start = 0;
-	   else if (exp.Find('$') != P_MAX_INDEX) start = 1;
+       if (exp.Find('^') != P_MAX_INDEX)  start = 1;
+	   else if (exp.Find('$') != P_MAX_INDEX) start = 0;
 	  
 	   if (start < 0) return s;   /// Logic error ignore rewrite
 
@@ -770,12 +773,39 @@ PString RewriteWildcard(
 	   int i = exp.Mid(c1+1,c2-c1-1).AsInteger();
 	   if (i == 0) return s;   /// Logic error ignore rewrite
 
-       if (b1 > 0) r = s.Left(i);
-	   else r = s.Mid(s.GetLength()-i);
+       if (start) rewrite = s.Left(i);
+	   else rewrite = s.Mid(s.GetLength()-i);
 	}
+	return l + rewrite + r;
+}
+
+
+PString ReplaceParameters(const PString & queryStr, const std::map<PString, PString> & queryParams)
+{
+	PString finalQuery = queryStr;
+	PINDEX pos = 0;
+	std::map<PString, PString>::const_iterator it;
+	PString var;
+
+	while (pos != P_MAX_INDEX) {
+		pos = finalQuery.Find('%', pos);
+		if (pos++ == P_MAX_INDEX)
+			break;
+		const char c = finalQuery[pos]; // char next after '%'
+		if (c == '{') {
+			const PINDEX closingBrace = finalQuery.Find('}', ++pos);
+			if (closingBrace != P_MAX_INDEX) {
+				PString var = finalQuery.Mid(pos,closingBrace-pos);
+				it = queryParams.find(var);
+				if (it!=queryParams.end()) {
+					finalQuery = finalQuery.Left(pos-2) + it->second + finalQuery.Mid(closingBrace+1);
+					pos = pos-2 + it->second.GetLength();
+				} else
+					pos = closingBrace;
+			}
+		}
+	}
+	return finalQuery;
+}
+
 
-    if (b1 > 0) 
-		return o.Left(b1-1) + r;
-	else
-		return r + o.Mid(b2+1);
-};
