@@ -13,7 +13,7 @@
 
 
 #ifndef H323UTIL_H
-#define H323UTIL_H "@(#) $Id: h323util.h,v 1.49 2012/12/23 06:39:46 shorne Exp $"
+#define H323UTIL_H "@(#) $Id: h323util.h,v 1.50 2013/04/17 05:29:58 shorne Exp $"
 
 #include <ptlib.h>
 #include <ptlib/sockets.h>
@@ -235,5 +235,16 @@ PString RewriteWildcard(
 	const PString & s,          /// original string to rewrite
 	const PString & expression /// prefix string that matched					
 	);
+
+/** ReplaceParameters 
+	General parameter rewrite procedure.
+	@return	Rewritten string.
+*/
+PString ReplaceParameters(
+	const PString & queryStr,				/// parametrized query string
+	const std::map<PString, PString> & queryParams	/// parameter values
+	);
+
+
 	
 #endif // H323UTIL_H
