@@ -26,7 +26,6 @@
 
 #include "h460/h4601.h"
 #include <h323pdu.h>
-#include <ptclib/delaychan.h>
 
 #define DEFAULT_PRESWORKER_TIMER 5   // fire thread every 5 seconds;
 
@@ -49,6 +48,9 @@ public:
 	// override from class PThread
 	virtual void Main();
 
+	// force the process thread to process messages
+	void ProcessNow();
+
 	// Close
 	void Close();
 
@@ -58,15 +60,12 @@ protected:
 private:
 	GkPresence *	handler;
 	long			waitTime;
-	bool			shutDown;
+	bool			processNow;
     PSyncPointAck	exitWorker;
-
-	PAdaptiveDelay m_delay;
-
 };
 
 PresWorker::PresWorker(GkPresence * _handler , int _waitTime)
-: PThread(5000, AutoDeleteThread), handler(_handler), waitTime(_waitTime), shutDown(false)
+: PThread(5000, AutoDeleteThread), handler(_handler), waitTime(_waitTime), processNow(false)
 {
 	PTRACE(4, "PRES\tPresence Thread instance fire every " << waitTime << " sec");
 	Resume();
@@ -79,23 +78,32 @@ PresWorker::~PresWorker()
 	
 void PresWorker::Main()
 {
+	int wint=100;
+	int wtime;
 	while (!exitWorker.Wait(0)) {
-		handler->DatabaseIncrementalUpdate();
-		if (shutDown) continue;
-			ProcessMessages();
-		if (shutDown) continue;
-			m_delay.Delay(waitTime);
+		wtime = 0;
+		ProcessMessages();
+		while (wtime < waitTime) {
+			wtime += wint;
+			PThread::Sleep(wint);
+			if (processNow)
+				break;
+		}
+		processNow = false;
 	}
 	exitWorker.Acknowledge();
 }
 
+void PresWorker::ProcessNow()
+{
+	processNow = true;
+}
+
 void PresWorker::Close()
 {
-	if (!shutDown) {
-		PTRACE(4, "PRES\tPresence Thread Shutdown");
-		shutDown = true;
-		exitWorker.Signal();
-	}
+	PTRACE(4, "PRES\tPresence Thread Shutdown");
+	ProcessNow();
+	exitWorker.Signal();
 }
 
 void BuildSCI(H225_RasMessage & sci_ras, PASN_OctetString & data)
@@ -144,12 +152,34 @@ void PresWorker::ProcessMessages()
 		while (i != epid.end()) {
 			endptr ep = RegistrationTable::Instance()->FindByEndpointId(*i);
 			if (ep) {
+#if H460P_VER > 2
+				bool dataToSend=false;
+				list<PASN_OctetString> element;
+				if (ep->HasPresenceData())
+					dataToSend = handler->BuildPresenceElement(H225_RasMessage::e_registrationRequest, *i, element);
+				else
+					dataToSend = handler->BuildPresenceElement(H225_RasMessage::e_serviceControlIndication, *i, element);
+
+				if (dataToSend) {
+					PASN_OctetString data;
+					H225_RasMessage sci_ras;
+					while (element.size() > 0) {
+						data = element.front();
+						element.pop_front();
+						BuildSCI(sci_ras,data);
+						RasServer::Instance()->SendRas(sci_ras, ep->GetRasAddress());
+						if (element.size() > 0)
+							PThread::Sleep(10);
+					}
+				}
+#else
 				PASN_OctetString element;
 				if (handler->BuildPresenceElement(H225_RasMessage::e_serviceControlIndication, *i, element)) {
 					H225_RasMessage sci_ras;
 					BuildSCI(sci_ras,element);
 					RasServer::Instance()->SendRas(sci_ras, ep->GetRasAddress());
 				}
+#endif
 			}
 			i++;
 		}
@@ -161,12 +191,27 @@ void PresWorker::ProcessMessages()
 		// Process the LRQ message for each TransportAddress
 		list<H225_TransportAddress>::iterator i = gkip.begin();
 		while (i != gkip.end()) {
+#if H460P_VER > 2
+			list<PASN_OctetString> element;
+			if (handler->BuildPresenceElement(H225_RasMessage::e_locationRequest,*i, element)) {
+				PASN_OctetString data;
+				H225_RasMessage lrq_ras;
+				while (element.size() > 0) {
+					data = element.front();
+					element.pop_front();
+					BuildLRQ(lrq_ras,data);
+					RasServer::Instance()->SendRas(lrq_ras, *i);
+					PThread::Sleep(10);
+				}
+			}
+#else
 			PASN_OctetString element;
 			if (handler->BuildPresenceElement(H225_RasMessage::e_locationRequest,*i, element)) {
 				H225_RasMessage lrq_ras;
 				BuildLRQ(lrq_ras,element);
 				RasServer::Instance()->SendRas(lrq_ras, *i);
 			}
+#endif
 			i++;
 		}
 	}
@@ -818,6 +863,7 @@ bool GkPresence::RegisterEndpoint(const H225_EndpointIdentifier & ep, const H225
 			}
 		}		
 	}
+	m_worker->ProcessNow();
 	return true;
 }
 
@@ -911,6 +957,21 @@ void GkPresence::ProcessPresenceElement(const PASN_OctetString & pdu)
 	}
 }
 
+#if H460P_VER > 2
+bool GkPresence::BuildPresenceElement(unsigned msgtag, const H225_EndpointIdentifier & ep, list<PASN_OctetString> & pdu)
+{
+	PWaitAndSignal m(m_AliasMutex);
+
+	return H323PresenceHandler::BuildPresenceElement(msgtag, ep, pdu);
+}
+
+bool GkPresence::BuildPresenceElement(unsigned msgtag, const H225_TransportAddress & ip, list<PASN_OctetString> & pdu)
+{
+	PWaitAndSignal m(m_AliasMutex);
+
+	return H323PresenceHandler::BuildPresenceElement(msgtag, ip, pdu);
+}
+#else
 bool GkPresence::BuildPresenceElement(unsigned msgtag, const H225_EndpointIdentifier & ep, PASN_OctetString & pdu)
 {
 	PWaitAndSignal m(m_AliasMutex);
@@ -924,6 +985,7 @@ bool GkPresence::BuildPresenceElement(unsigned msgtag, const H225_TransportAddre
 
 	return H323PresenceHandler::BuildPresenceElement(msgtag, ip, pdu);
 }
+#endif
 
 bool GkPresence::EnQueueFullNotification(const H225_AliasAddress & local, const H225_AliasAddress & remote)
 {
@@ -1425,7 +1487,7 @@ bool GkPresence::RemoveSubscription(unsigned type,const H460P_PresenceIdentifier
 
 		H323PresenceStore::iterator it2 = localStore.find(it->second.m_Alias);
 		if (it2 != localStore.end())
-			HandleStatusUpdates(pid,it2->first,type,it->second.m_subscriber);
+			HandleStatusUpdates(pid,it2->first, type,it->second.m_subscriber);
 
 	   PTRACE(4, "PRES\tRemoved Subscription " << PresMsgType(type) << " : " << it->second.m_subscriber << " to " << it->second.m_Alias);
 	   return true;
@@ -1571,7 +1633,7 @@ void GkPresence::OnSubscription(MsgType tag, const H460P_PresenceSubscription &
 		H460P_PresencePDU msg;
 		BuildSubscriptionMsg(subscription,msg);
 		if (!HandleForwardPresence(subscription.m_identifier, msg)) {
-			PTRACE(2, "PRES\tSubscription received " << subscription.m_identifier.m_guid << " from " << AsString(addr,0) << " not handled");
+			PTRACE(2, "PRES\tSubscription received " << AsString(subscription.m_identifier.m_guid) << " from " << AsString(addr,0) << " not handled");
 			return;
 		}
 	}
