@@ -2,7 +2,7 @@
 //
 // pwlib_compat.h PWLib compatibility header
 //
-// Copyright (c) 2006-2012, Jan Willamowius
+// Copyright (c) 2006-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 
 
 #ifndef PWLIB_COMPAT_H
-#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.75 2012/12/28 20:37:05 willamowius Exp $"
+#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.80 2013/04/29 18:04:47 shorne Exp $"
 
 #include "ptbuildopts.h"
 #include "openh323buildopts.h"
@@ -119,11 +119,21 @@
 		#if PTLIB_BUILD >= 6
 			#define hasLDAPStartTLS 1
 		#endif
+		#ifdef _WIN32
+			#define hasWorkerDeleteBug	1
+		#endif
+	#endif
+	#if (PTLIB_MINOR == 10) || (PTLIB_MINOR == 11)
+		#define hasThreadAutoDeleteBug	1
 	#endif
 	#if PTLIB_MINOR >= 11
 		#define hasLDAPStartTLS 1
         #define hasNewSTUN      1
         #define hasPTRACE2      1
+        #define hasAutoCreateAuthenticators	1
+	#endif
+	#if PTLIB_MINOR >= 12
+		#define hasNoMutexWillBlock	1
 	#endif
 	// bug with no trailing NULL bytes in BMP strings, fixed in PTLib 2.7.1
 	#if ((PTLIB_MINOR == 2) || (PTLIB_MINOR == 4 && PTLIB_BUILD <= 5) || (PTLIB_MINOR == 5 && PTLIB_BUILD <= 2) || (PTLIB_MINOR == 6 && PTLIB_BUILD <= 4))
@@ -159,12 +169,16 @@
 #endif
 
 ///////////////////////////////////////////////
-
 // OpenH323/H323Plus version matching
+
 #ifdef H323_H235
 	#define HAS_H235_MEDIA 1      // H.235.6 Media Encryption Support
 #endif
 
+#ifdef H323_H450
+	#define HAS_H450 1
+#endif
+
 #ifdef H323_H460
 	#define HAS_H460 1    // H460 support
 #else
@@ -181,6 +195,9 @@
 
 #ifdef HAS_H46023		// config switch if H.460.23 should be enabled
 	#ifdef H323_H46023	// feature detection, if library supports H.460.23
+		#ifdef H323_H46024A
+			#define HAS_H46024A	1
+		#endif
 		#ifdef H323_H46024B
 			#define HAS_H46024B	1
 		#endif
@@ -189,9 +206,12 @@
 
 #ifdef H323_H460P
 	#define HAS_H460P	1  // Presence
-//#ifdef H323_H460P_VER
-	#define H460P_VER 2 // H323_H460P_VER
-	#define OID3 "1.3.6.1.4.1.17090.0.12"  // Presence v2
+#if H323PLUS_VER > 1251
+	#define H460P_VER 3
+#else
+	#define H460P_VER 2
+#endif
+	#define OID3 "1.3.6.1.4.1.17090.0.12"  // Presence v2+
 //#else
 //	#define H460P_VER 1
 //	#define OID3 "1.3.6.1.4.1.17090.0.3"  // Presence
@@ -229,12 +249,8 @@
 	#endif	
 #endif
 
-#if defined(P_SNMP) || defined(HAS_NETSNMP) || defined(_WIN32)
+#if defined(HAS_PTLIBSNMP) || defined(HAS_NETSNMP) || defined(HAS_WINSNMP)
 #define HAS_SNMP 1
 #endif
 
-#if defined(H323_H450)
-#define HAS_H450 1
-#endif
-
 #endif // PWLIB_COMPAT_H
