@@ -21,6 +21,12 @@
 #include "snmp.h"
 #include "gk.h"
 
+#ifdef _WIN32
+#	ifndef SHUT_RDWR
+#   	define SHUT_RDWR SD_BOTH
+#	endif
+#endif
+
 using std::mem_fun;
 using std::bind1st;
 using std::partition;
@@ -141,11 +147,12 @@ bool YaSocket::Close()
 	int handle = os_handle;
 	os_handle = -1;
 	::shutdown(handle, SHUT_RDWR);
-#ifdef _WIN32
-	::closesocket(handle);
-#else
+	// reset linger, so close() won't wait 3 seconds if remote doesn't respond
+	struct linger so_linger;
+	so_linger.l_onoff = 0;
+	so_linger.l_linger = 0;
+	::setsockopt(handle, SOL_SOCKET, SO_LINGER, &so_linger, sizeof(so_linger));
 	::close(handle);
-#endif
 	return true;
 }
 
@@ -761,7 +768,7 @@ bool UDPSocket::DualStackListen(const PIPSocket::Address & localAddr, WORD newPo
 	return true;
 }
 
-#endif
+#endif	// hasIPV6
 
 bool SocketSelectList::Select(SelectType t, const PTimeInterval & timeout)
 {
@@ -849,10 +856,10 @@ bool USocket::WriteData(const BYTE * buf, int len)
 		
 	int remaining = len;
 	if (qsize == 0 && 
-#if PTLIB_VER < 2120
-		!writeMutex.WillBlock()
-#else
+#ifdef hasNoMutexWillBlock
 		writeMutex.Try()
+#else
+		!writeMutex.WillBlock()
 #endif
 		) {
 		PWaitAndSignal lock(writeMutex);
@@ -1124,14 +1131,14 @@ bool TCPServer::CloseListener(TCPListenSocket * socket)
 
 void TCPServer::ReadSocket(IPSocket * socket)
 {
-#if PTLIB_VER < 2120
-	if (ShutdownMutex.WillBlock()) {
-#else
+#ifdef hasNoMutexWillBlock
 	if (!ShutdownMutex.Wait(0)) {
+#else
+	if (ShutdownMutex.WillBlock()) {
 #endif
 		PTRACE(4, GetName() << "\tShutdown: Rejecting call on " << socket->GetName());
 		int rej = ::accept(socket->GetHandle(), NULL, NULL);
-		::shutdown(rej, 2 /* SHUT_RDWR */ );
+		::shutdown(rej, SHUT_RDWR );
 #if defined(_WIN32)
 		::closesocket(rej);
 #else
